@@ -38,9 +38,9 @@
  *                 Guilhem Tardy (gtardy@salyens.com)
  *                 Craig Southeren (craigs@postincrement.com)
  *
- * $Revision: 28125 $
+ * $Revision: 28438 $
  * $Author: rjongbloed $
- * $Date: 2012-07-31 23:55:23 -0500 (Tue, 31 Jul 2012) $
+ * $Date: 2012-10-02 00:34:04 -0500 (Tue, 02 Oct 2012) $
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
@@ -251,7 +251,7 @@ bool H263_Base_EncoderContext::Init(CodecID codecId)
     m_context->debug |= FF_DEBUG_BUGS | FF_DEBUG_BUFFERS;
 #endif
 
-  PTRACE(4, m_prefix, "Encoder created (SVN $Revision: 28125 $)");
+  PTRACE(4, m_prefix, "Encoder created (SVN $Revision: 28438 $)");
 
   return true;
 }
@@ -682,7 +682,7 @@ H263_Base_DecoderContext::H263_Base_DecoderContext(const char * prefix, Depacket
 
   m_depacketizer->NewFrame();
 
-  PTRACE(4, m_prefix, "Decoder created (SVN $Revision: 28125 $)");
+  PTRACE(4, m_prefix, "Decoder created (SVN $Revision: 28438 $)");
 }
 
 H263_Base_DecoderContext::~H263_Base_DecoderContext()
@@ -772,13 +772,9 @@ bool H263_Base_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen,
   if (m_depacketizer->IsIntraFrame())
     flags |= PluginCodec_ReturnCoderIFrame;
 
-#ifndef FFMPEG_HAS_DECODE_ERROR_COUNT
-  // We have repuposed this variable as a count of decode errors
-  // detected by the FFMPEG logging intercept.
-  #define  decode_error_count  error_rate
-#endif
-
+#ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
   int error_before = m_context->decode_error_count;
+#endif
 
   PTRACE(5, m_prefix, "Decoding " << m_depacketizer->GetLength()  << " bytes");
   int gotPicture = 0;
@@ -790,9 +786,11 @@ bool H263_Base_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen,
 
   m_depacketizer->NewFrame();
 
+#ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
   if (error_before != m_context->decode_error_count)
     flags = PluginCodec_ReturnCoderRequestIFrame;
+#endif
 
   if (!gotPicture) {
     PTRACE(3, m_prefix, "Decoded "<< bytesDecoded << " bytes without getting a Picture"); 
