@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25967 $
+ * $Revision: 28048 $
  * $Author: rjongbloed $
- * $Date: 2011-06-02 23:33:56 -0500 (Thu, 02 Jun 2011) $
+ * $Date: 2012-07-17 22:41:57 -0500 (Tue, 17 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -187,46 +187,6 @@ const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
 };
 
 
-const OpalMediaFormat & GetOpalRFC2833()
-{
-  static struct OpalRFC2833MediaFormat : public OpalMediaFormat {
-    OpalRFC2833MediaFormat()
-      : OpalMediaFormat(OPAL_RFC2833,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
-                        "telephone-event",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "0-16,32,36"));
-    }
-  } const RFC2833;
-  return RFC2833;
-}
-
-const OpalMediaFormat & GetOpalCiscoNSE()
-{
-  static struct OpalCiscoNSEMediaFormat : public OpalMediaFormat {
-    OpalCiscoNSEMediaFormat()
-      : OpalMediaFormat(OPAL_CISCONSE,
-                        "userinput",
-                        (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
-                        "NSE",
-                        true,   // Needs jitter
-                        32*(1000/50), // bits/sec  (32 bits every 50ms)
-                        4,      // bytes/frame
-                        10*8,   // 10 millisecond
-                        OpalMediaFormat::AudioClockRate)
-    {
-      AddOption(new OpalMediaOptionString("FMTP", false, "192,193"));
-    }
-  } const CiscoNSE;
-  return CiscoNSE;
-}
-
 static OpalMediaFormatList & GetMediaFormatsList()
 {
   static class OpalMediaFormatListMaster : public OpalMediaFormatList
@@ -763,8 +723,8 @@ const PString & OpalMediaFormat::MediaPacketizationOption()  { static const PCon
 const PString & OpalMediaFormat::MediaPacketizationsOption() { static const PConstString s(PLUGINCODEC_MEDIA_PACKETIZATIONS); return s; }
 #endif
 
-const PString & OpalMediaFormat::ProtocolOption()        { static const PConstString s("Protocol");           return s; }
-const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static const PConstString s("Max Tx Packet Size"); return s; }
+const PString & OpalMediaFormat::ProtocolOption()        { static const PConstString s(PLUGINCODEC_OPTION_PROTOCOL); return s; }
+const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static const PConstString s(PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE); return s; }
 
 OpalMediaFormat::OpalMediaFormat(OpalMediaFormatInternal * info)
   : m_info(NULL)
@@ -814,6 +774,28 @@ OpalMediaFormat::OpalMediaFormat(const char * fullName,
 }
 
 
+OpalMediaFormat::OpalMediaFormat(const OpalMediaFormat & c)
+  : PContainer() // can't use PContainer copy c-tor as this c-tor must be synchronized
+  , m_info(NULL)
+{
+  PWaitAndSignal m(c.m_mutex); // here is no need to use mutex of the shared object
+  PContainer::AssignContents(c);
+  m_info = c.m_info;
+}
+
+
+OpalMediaFormat::~OpalMediaFormat()
+{
+  if (m_info != NULL)
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+
+  Destruct();
+
+  if (m_info != NULL)
+    m_info->media_format_mutex.Signal();
+}
+
+
 void OpalMediaFormat::Construct(OpalMediaFormatInternal * info)
 {
   if (info == NULL)
@@ -893,12 +875,21 @@ PBoolean OpalMediaFormat::MakeUnique()
 
 void OpalMediaFormat::AssignContents(const PContainer & c)
 {
-  m_mutex.Wait();
+  PWaitAndSignal m1(m_mutex);
 
-  PContainer::AssignContents(c);
-  m_info = ((const OpalMediaFormat &)c).m_info;
+  const OpalMediaFormat & other = (const OpalMediaFormat &)c;
+  PWaitAndSignal m2(other.m_mutex);
 
-  m_mutex.Signal();
+  if (m_info != NULL) {
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+    PContainer::AssignContents(c);
+    if (m_info != NULL)
+      m_info->media_format_mutex.Signal();
+  }
+  else // current object can't be removed
+    PContainer::AssignContents(c);
+
+  m_info = other.m_info;
 }
 
 
@@ -907,8 +898,8 @@ void OpalMediaFormat::DestroyContents()
   m_mutex.Wait();
 
   if (m_info != NULL) {
-    m_info->media_format_mutex.Wait();
     delete m_info;
+    m_info = NULL;
   }
 
   m_mutex.Signal();
@@ -1005,6 +996,30 @@ bool OpalMediaFormat::ValidateMerge(const OpalMediaFormat & mediaFormat) const
 }
 
 
+#if OPAL_H323
+PStringSet OpalMediaFormat::GetMediaPacketizations() const
+{
+  return GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                         GetOptionString(OpalMediaFormat::MediaPacketizationOption())).Tokenise(",");
+}
+
+
+void OpalMediaFormat::SetMediaPacketizations(const PStringSet & packetizations)
+{
+  if (packetizations.IsEmpty()) {
+    SetOptionString(MediaPacketizationsOption(), PString::Empty());
+    SetOptionString(MediaPacketizationOption(),  PString::Empty());
+  }
+  else {
+    PStringStream strm;
+    strm << setfill(',') << packetizations;
+    SetOptionString(MediaPacketizationsOption(), strm);
+    SetOptionString(MediaPacketizationOption(),  packetizations.GetKeyAt(0));
+  }
+}
+#endif
+
+
 OpalMediaFormatList OpalMediaFormat::GetAllRegisteredMediaFormats()
 {
   OpalMediaFormatList copy;
@@ -1043,6 +1058,22 @@ bool OpalMediaFormat::SetRegisteredMediaFormat(const OpalMediaFormat & mediaForm
 }
 
 
+bool OpalMediaFormat::RemoveRegisteredMediaFormat(const OpalMediaFormat & mediaFormat)
+{
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  for (OpalMediaFormatList::iterator format = registeredFormats.begin(); format != registeredFormats.end(); ++format) {
+    if (*format == mediaFormat) {
+      registeredFormats.erase(format);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
@@ -1626,7 +1657,7 @@ const PString & OpalVideoFormat::MaxRxFrameWidthOption()          { static const
 const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static const PConstString s(PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT);       return s; }
 const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static const PConstString s(PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF);return s; }
 const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static const PConstString s(PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD);       return s; }
-const PString & OpalVideoFormat::RateControlEnableOption()        { static const PConstString s("Rate Control Enable");                        return s; }
+const PString & OpalVideoFormat::RateControlPeriodOption()        { static const PConstString s(PLUGINCODEC_OPTION_RATE_CONTROL_PERIOD);       return s; }
 const PString & OpalVideoFormat::RateControllerOption()           { static const PConstString s("Rate Controller");                            return s; }
 const PString & OpalVideoFormat::ContentRoleOption()              { static const PConstString s("Content Role");                               return s; }
 const PString & OpalVideoFormat::ContentRoleMaskOption()          { static const PConstString s("Content Role Mask");                          return s; }
@@ -1678,8 +1709,8 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameHeightOption(),         false, OpalMediaOption::MinMerge,    maxFrameHeight,              16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,    1000));
-  AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          false, OpalMediaOption::NoMerge,     8192,                        100,  8192));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlPeriodOption(),        false, OpalMediaOption::AlwaysMerge, 1000,                        100, 60000));
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          true,  OpalMediaOption::AlwaysMerge, PluginCodec_RTP_MaxPayloadSize, 100    ));
   AddOption(new OpalMediaOptionString  (OpalVideoFormat::RateControllerOption(),           false                                                                       ));
 
 
@@ -1751,6 +1782,23 @@ OpalMediaFormatList::OpalMediaFormatList(const OpalMediaFormat & format)
 }
 
 
+OpalMediaFormatList & OpalMediaFormatList::operator+=(const PString & wildcard)
+{
+  MakeUnique();
+
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  OpalMediaFormatList::const_iterator fmt;
+  while ((fmt = registeredFormats.FindFormat(wildcard, fmt)) != registeredFormats.end()) {
+    if (!HasFormat(*fmt))
+      OpalMediaFormatBaseList::Append(fmt->Clone());
+  }
+
+  return *this;
+}
+
+
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & format)
 {
   MakeUnique();
@@ -1796,11 +1844,39 @@ void OpalMediaFormatList::Remove(const PStringArray & maskList)
 
   PTRACE(4,"MediaFormat\tRemoving codecs " << setfill(',') << maskList);
 
-  for (PINDEX i = 0; i < maskList.GetSize(); i++) {
-    OpalMediaFormatList::const_iterator fmt;
-    while ((fmt = FindFormat(maskList[i])) != end())
-      erase(fmt);
+  PINDEX i;
+  PStringList notMasks;
+  const_iterator fmt;
+
+  for (i = 0; i < maskList.GetSize(); i++) {
+    PString mask = maskList[i];
+    if (mask[0] == '!')
+      notMasks.AppendString(mask);
+    else {
+      while ((fmt = FindFormat(mask)) != end())
+        erase(fmt);
+    }
   }
+
+  switch (notMasks.GetSize()) {
+    case 0 :
+      return;
+
+    case 1 :
+      while ((fmt = FindFormat(notMasks[0])) != end())
+        erase(fmt);
+      return;
+  }
+
+  OpalMediaFormatList formatsToKeep;
+  for (i = 0; i < notMasks.GetSize(); i++) {
+    PString keeper = notMasks[i].Mid(1);
+    fmt = const_iterator();
+    while ((fmt = FindFormat(keeper, fmt)) != end())
+      formatsToKeep += *fmt;
+  }
+
+  *this = formatsToKeep;
 }
 
 
