@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28162 $
+ * $Revision: 28428 $
  * $Author: rjongbloed $
- * $Date: 2012-08-07 22:59:15 -0500 (Tue, 07 Aug 2012) $
+ * $Date: 2012-10-01 03:38:50 -0500 (Mon, 01 Oct 2012) $
  *
  * This code implements all or part of the following RFCs
  *
@@ -75,7 +75,8 @@ ostream & operator<<(ostream & strm, SIPHandler::State state)
 
 SIPHandler::SIPHandler(SIP_PDU::Methods method, SIPEndPoint & ep, const SIPParameters & params)
   : endpoint(ep)
-  , authentication(NULL)
+  , m_authentication(NULL)
+  , m_authenticateErrors(0)
   , m_username(params.m_authID)
   , m_password(params.m_password)
   , m_realm(params.m_realm)
@@ -111,7 +112,7 @@ SIPHandler::~SIPHandler()
     delete m_transport;
   }
 
-  delete authentication;
+  delete m_authentication;
 
   PTRACE(4, "SIP\tDestroyed " << m_method << " handler for " << m_addressOfRecord);
 }
@@ -375,9 +376,9 @@ bool SIPHandler::WriteSIPHandler(OpalTransport & transport, bool forked)
   if (GetState() == Unsubscribing)
     mime.SetExpires(0);
 
-  if (authentication != NULL) {
+  if (m_authentication != NULL) {
     SIPAuthenticator auth(*transaction);
-    authentication->Authorise(auth); // If already have info from last time, use it!
+    m_authentication->Authorise(auth); // If already have info from last time, use it!
   }
 
   if (transaction->Start()) {
@@ -421,7 +422,7 @@ void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & resp
     case SIP_PDU::Failure_UnAuthorised :
     case SIP_PDU::Failure_ProxyAuthenticationRequired :
       OnReceivedAuthenticationRequired(transaction, response);
-      break;
+      return;
 
     case SIP_PDU::Failure_IntervalTooBrief :
       OnReceivedIntervalTooBrief(transaction, response);
@@ -437,6 +438,8 @@ void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & resp
       else
         OnFailed(response);
   }
+
+  m_authenticateErrors = 0;
 }
 
 
@@ -458,38 +461,24 @@ void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction
 
 void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
 {
-  bool isProxy = response.GetStatusCode() == SIP_PDU::Failure_ProxyAuthenticationRequired;
-
-  PTRACE(3, "SIP\tReceived " << (isProxy ? "Proxy " : "")
-         << "Authentication Required response, state=" << GetState());
-  
-  // authenticate 
-  PString errorMsg;
-  SIPAuthentication * newAuth = PHTTPClientAuthentication::ParseAuthenticationRequired(isProxy, response.GetMIME(), errorMsg);
-  if (newAuth == NULL) {
-    PTRACE(2, "SIP\t" << errorMsg);
-    OnFailed(SIP_PDU::Failure_Forbidden);
-    return;
-  }
-
   // If either username or password blank, try and fine values from other
   // handlers which might be logged into the realm, or the proxy, if one.
-  if (!endpoint.GetAuthentication(*newAuth, authentication, GetProxy(), m_username, m_password)) {
-    delete newAuth;
-
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
+  SIP_PDU::StatusCodes status = endpoint.HandleAuthentication(m_authentication,
+                                                              m_authenticateErrors,
+                                                              response,
+                                                              GetProxy(),
+                                                              m_username,
+                                                              m_password);
+  if (status != SIP_PDU::Successful_OK) {
+    OnFailed(status);
     if (GetState() != Unsubscribing && !transaction.IsCanceled())
       RetryLater(m_offlineExpireTime);
     return;
   }
 
-  // switch authentication schemes
-  delete authentication;
-  authentication = newAuth;
-
   // If we changed realm (or hadn't got one yet) update the handler database
-  if (m_realm != newAuth->GetAuthRealm()) {
-    m_realm = newAuth->GetAuthRealm();
+  if (m_realm != m_authentication->GetAuthRealm()) {
+    m_realm = m_authentication->GetAuthRealm();
     PTRACE(3, "SIP\tAuth realm set to " << m_realm);
     endpoint.UpdateHandlerIndexes(this);
   }
